From: Peter Zijlstra <a.p.zijlstra@chello.nl>
To: Chris Mason <chris.mason@oracle.com>,
Frank Rowand <frank.rowand@am.sony.com>,
Ingo Molnar <mingo@elte.hu>, Thomas Gleixner <tglx@linutronix.de>,
Mike Galbraith <efault@gmx.de>, Oleg Nesterov <oleg@redhat.com>,
Paul Turner <pjt@google.com>, Jens Axboe <axboe@kernel.dk>,
Yong Zhang <yong.zhang0@gmail.com>
Cc: linux-kernel@vger.kernel.org, Peter Zijlstra <a.p.zijlstra@chello.nl>
Subject: [PATCH 07/21] sched: Serialize p->cpus_allowed and ttwu() using p->pi_lock
Date: Tue, 05 Apr 2011 17:23:45 +0200 [thread overview]
Message-ID: <20110405152728.990364093@chello.nl> (raw)
In-Reply-To: 20110405152338.692966333@chello.nl
[-- Attachment #1: sched-pi_lock-wakeup.patch --]
[-- Type: text/plain, Size: 3668 bytes --]
Currently p->pi_lock already serializes p->sched_class, also put
p->cpus_allowed and try_to_wake_up() under it, this prepares the way
to do the first part of ttwu() without holding rq->lock.
By having p->sched_class and p->cpus_allowed serialized by p->pi_lock,
we prepare the way to call select_task_rq() without holding rq->lock.
Signed-off-by: Peter Zijlstra <a.p.zijlstra@chello.nl>
Reviewed-by: Frank Rowand <frank.rowand@am.sony.com>
---
kernel/sched.c | 37 ++++++++++++++++---------------------
1 file changed, 16 insertions(+), 21 deletions(-)
Index: linux-2.6/kernel/sched.c
===================================================================
--- linux-2.6.orig/kernel/sched.c
+++ linux-2.6/kernel/sched.c
@@ -2301,7 +2301,7 @@ void task_oncpu_function_call(struct tas
#ifdef CONFIG_SMP
/*
* ->cpus_allowed is protected by either TASK_WAKING or rq->lock held.
*/
static int select_fallback_rq(int cpu, struct task_struct *p)
{
@@ -2334,7 +2334,7 @@ static int select_fallback_rq(int cpu, s
}
/*
* The caller (fork, wakeup) owns TASK_WAKING, ->cpus_allowed is stable.
*/
static inline
int select_task_rq(struct rq *rq, struct task_struct *p, int sd_flags, int wake_flags)
@@ -2450,7 +2450,8 @@ static int try_to_wake_up(struct task_st
this_cpu = get_cpu();
smp_wmb();
- rq = task_rq_lock(p, &flags);
+ raw_spin_lock_irqsave(&p->pi_lock, flags);
+ rq = __task_rq_lock(p);
if (!(p->state & state))
goto out;
@@ -2508,7 +2509,8 @@ static int try_to_wake_up(struct task_st
ttwu_stat(rq, p, cpu, wake_flags);
success = 1;
out:
- task_rq_unlock(rq, &flags);
+ __task_rq_unlock(rq);
+ raw_spin_unlock_irqrestore(&p->pi_lock, flags);
put_cpu();
return success;
@@ -4543,6 +4545,8 @@ void rt_mutex_setprio(struct task_struct
BUG_ON(prio < 0 || prio > MAX_PRIO);
+ lockdep_assert_held(&p->pi_lock);
+
rq = task_rq_lock(p, &flags);
trace_sched_pi_setprio(p, prio);
@@ -5150,7 +5154,6 @@ long sched_getaffinity(pid_t pid, struct
{
struct task_struct *p;
unsigned long flags;
- struct rq *rq;
int retval;
get_online_cpus();
@@ -5165,9 +5168,9 @@ long sched_getaffinity(pid_t pid, struct
if (retval)
goto out_unlock;
- rq = task_rq_lock(p, &flags);
+ raw_spin_lock_irqsave(&p->pi_lock, flags);
cpumask_and(mask, &p->cpus_allowed, cpu_online_mask);
- task_rq_unlock(rq, &flags);
+ raw_spin_unlock_irqrestore(&p->pi_lock, flags);
out_unlock:
rcu_read_unlock();
@@ -5652,18 +5655,8 @@ int set_cpus_allowed_ptr(struct task_str
unsigned int dest_cpu;
int ret = 0;
- /*
- * Serialize against TASK_WAKING so that ttwu() and wunt() can
- * drop the rq->lock and still rely on ->cpus_allowed.
- */
-again:
- while (task_is_waking(p))
- cpu_relax();
- rq = task_rq_lock(p, &flags);
- if (task_is_waking(p)) {
- task_rq_unlock(rq, &flags);
- goto again;
- }
+ raw_spin_lock_irqsave(&p->pi_lock, flags);
+ rq = __task_rq_lock(p);
if (!cpumask_intersects(new_mask, cpu_active_mask)) {
ret = -EINVAL;
@@ -5691,13 +5684,15 @@ int set_cpus_allowed_ptr(struct task_str
if (migrate_task(p, rq)) {
struct migration_arg arg = { p, dest_cpu };
/* Need help from migration thread: drop lock and wait. */
- task_rq_unlock(rq, &flags);
+ __task_rq_unlock(rq);
+ raw_spin_unlock_irqrestore(&p->pi_lock, flags);
stop_one_cpu(cpu_of(rq), migration_cpu_stop, &arg);
tlb_migrate_finish(p->mm);
return 0;
}
out:
- task_rq_unlock(rq, &flags);
+ __task_rq_unlock(rq);
+ raw_spin_unlock_irqrestore(&p->pi_lock, flags);
return ret;
}
next prev parent reply other threads:[~2011-04-05 15:31 UTC|newest]
Thread overview: 152+ messages / expand[flat|nested] mbox.gz Atom feed top
2011-04-05 15:23 [PATCH 00/21] sched: Reduce runqueue lock contention -v6 Peter Zijlstra
2011-04-05 15:23 ` [PATCH 01/21] sched: Provide scheduler_ipi() callback in response to smp_send_reschedule() Peter Zijlstra
2011-04-13 21:15 ` Tony Luck
2011-04-13 21:38 ` Peter Zijlstra
2011-04-14 8:31 ` [tip:sched/locking] " tip-bot for Peter Zijlstra
2011-04-05 15:23 ` [PATCH 02/21] sched: Always provide p->on_cpu Peter Zijlstra
2011-04-14 8:31 ` [tip:sched/locking] " tip-bot for Peter Zijlstra
2011-04-05 15:23 ` [PATCH 03/21] mutex: Use p->on_cpu for the adaptive spin Peter Zijlstra
2011-04-14 8:32 ` [tip:sched/locking] " tip-bot for Peter Zijlstra
2011-04-05 15:23 ` [PATCH 04/21] sched: Change the ttwu success details Peter Zijlstra
2011-04-13 9:23 ` Peter Zijlstra
2011-04-13 10:48 ` Peter Zijlstra
2011-04-13 11:06 ` Peter Zijlstra
2011-04-13 18:39 ` Tejun Heo
2011-04-13 19:11 ` Peter Zijlstra
2011-04-14 8:32 ` [tip:sched/locking] sched: Change the ttwu() " tip-bot for Peter Zijlstra
2011-04-05 15:23 ` [PATCH 05/21] sched: Clean up ttwu stats Peter Zijlstra
2011-04-14 8:33 ` [tip:sched/locking] sched: Clean up ttwu() stats tip-bot for Peter Zijlstra
2011-04-05 15:23 ` [PATCH 06/21] sched: Provide p->on_rq Peter Zijlstra
2011-04-14 8:33 ` [tip:sched/locking] " tip-bot for Peter Zijlstra
2011-04-05 15:23 ` Peter Zijlstra [this message]
2011-04-14 8:34 ` [tip:sched/locking] sched: Serialize p->cpus_allowed and ttwu() using p->pi_lock tip-bot for Peter Zijlstra
2011-04-05 15:23 ` [PATCH 08/21] sched: Drop the rq argument to sched_class::select_task_rq() Peter Zijlstra
2011-04-14 8:34 ` [tip:sched/locking] " tip-bot for Peter Zijlstra
2011-04-05 15:23 ` [PATCH 09/21] sched: Remove rq argument to sched_class::task_waking() Peter Zijlstra
2011-04-14 8:35 ` [tip:sched/locking] " tip-bot for Peter Zijlstra
2011-04-05 15:23 ` [PATCH 10/21] sched: Deal with non-atomic min_vruntime reads on 32bits Peter Zijlstra
2011-04-14 8:35 ` [tip:sched/locking] " tip-bot for Peter Zijlstra
2011-04-05 15:23 ` [PATCH 11/21] sched: Delay task_contributes_to_load() Peter Zijlstra
2011-04-14 8:35 ` [tip:sched/locking] " tip-bot for Peter Zijlstra
2011-04-05 15:23 ` [PATCH 12/21] sched: Also serialize ttwu_local() with p->pi_lock Peter Zijlstra
2011-04-14 8:36 ` [tip:sched/locking] " tip-bot for Peter Zijlstra
2011-04-05 15:23 ` [PATCH 13/21] sched: Add p->pi_lock to task_rq_lock() Peter Zijlstra
2011-04-14 8:36 ` [tip:sched/locking] " tip-bot for Peter Zijlstra
2011-06-01 13:58 ` Arne Jansen
2011-06-01 16:35 ` Peter Zijlstra
2011-06-01 17:20 ` Arne Jansen
2011-06-01 18:09 ` Peter Zijlstra
2011-06-01 18:44 ` Peter Zijlstra
2011-06-01 19:30 ` Arne Jansen
2011-06-01 21:09 ` Linus Torvalds
2011-06-03 9:15 ` Peter Zijlstra
2011-06-03 10:02 ` Arne Jansen
2011-06-03 10:30 ` Peter Zijlstra
2011-06-03 11:52 ` Arne Jansen
2011-06-05 8:17 ` Ingo Molnar
2011-06-05 8:53 ` Arne Jansen
2011-06-05 9:41 ` Ingo Molnar
2011-06-05 9:45 ` Ingo Molnar
2011-06-05 9:43 ` Arne Jansen
2011-06-05 9:55 ` Ingo Molnar
2011-06-05 10:22 ` Arne Jansen
2011-06-05 11:01 ` Ingo Molnar
2011-06-05 11:19 ` [debug patch] printk: Add a printk killswitch to robustify NMI watchdog messages Ingo Molnar
2011-06-05 11:36 ` Ingo Molnar
2011-06-05 11:57 ` Arne Jansen
2011-06-05 13:39 ` Ingo Molnar
2011-06-05 13:54 ` Arne Jansen
2011-06-05 14:06 ` Ingo Molnar
2011-06-05 14:45 ` Arne Jansen
2011-06-05 14:10 ` Ingo Molnar
2011-06-05 14:31 ` Arne Jansen
2011-06-05 15:13 ` Ingo Molnar
2011-06-05 15:26 ` Ingo Molnar
2011-06-05 15:32 ` Ingo Molnar
2011-06-05 16:07 ` Arne Jansen
2011-06-05 16:35 ` Arne Jansen
2011-06-05 16:50 ` Arne Jansen
2011-06-05 17:20 ` Ingo Molnar
2011-06-05 17:42 ` Arne Jansen
2011-06-05 18:59 ` Ingo Molnar
2011-06-05 19:30 ` Arne Jansen
2011-06-05 19:44 ` Ingo Molnar
2011-06-05 20:15 ` Arne Jansen
2011-06-06 6:56 ` Arne Jansen
2011-06-06 9:01 ` Peter Zijlstra
2011-06-06 9:18 ` Arne Jansen
2011-06-06 9:24 ` Peter Zijlstra
2011-06-06 9:52 ` Peter Zijlstra
2011-06-06 10:00 ` Arne Jansen
2011-06-06 10:26 ` Peter Zijlstra
2011-06-06 13:25 ` Peter Zijlstra
2011-06-06 15:04 ` Ingo Molnar
2011-06-06 15:08 ` Ingo Molnar
2011-06-06 17:44 ` Mike Galbraith
2011-06-07 5:20 ` Mike Galbraith
2011-06-06 13:10 ` Ingo Molnar
2011-06-06 13:12 ` Peter Zijlstra
2011-06-06 13:21 ` Ingo Molnar
2011-06-06 13:31 ` Peter Zijlstra
2011-06-06 7:34 ` Arne Jansen
2011-06-05 15:34 ` Arne Jansen
2011-06-06 8:38 ` Peter Zijlstra
2011-06-06 14:58 ` Ingo Molnar
2011-06-06 15:09 ` Peter Zijlstra
2011-06-06 15:47 ` Peter Zijlstra
2011-06-06 15:52 ` Ingo Molnar
2011-06-06 16:00 ` Peter Zijlstra
2011-06-06 16:08 ` Ingo Molnar
2011-06-06 16:12 ` Peter Zijlstra
2011-06-06 16:17 ` Ingo Molnar
2011-06-06 16:38 ` Arne Jansen
2011-06-06 16:45 ` Arne Jansen
2011-06-06 16:53 ` Peter Zijlstra
2011-06-06 17:07 ` Ingo Molnar
2011-06-06 17:11 ` Peter Zijlstra
2011-06-08 15:50 ` Peter Zijlstra
2011-06-08 19:17 ` Ingo Molnar
2011-06-08 19:27 ` Linus Torvalds
2011-06-08 20:32 ` Peter Zijlstra
2011-06-08 20:53 ` Linus Torvalds
2011-06-08 20:54 ` Thomas Gleixner
2011-06-08 19:45 ` Peter Zijlstra
2011-06-08 20:52 ` Ingo Molnar
2011-06-08 21:49 ` Peter Zijlstra
2011-06-08 21:57 ` Thomas Gleixner
2011-06-06 16:44 ` Peter Zijlstra
2011-06-06 16:50 ` Peter Zijlstra
2011-06-06 17:13 ` Ingo Molnar
2011-06-06 17:04 ` Peter Zijlstra
2011-06-06 17:11 ` Ingo Molnar
2011-06-06 17:57 ` Arne Jansen
2011-06-06 18:07 ` Ingo Molnar
2011-06-06 18:14 ` Arne Jansen
2011-06-06 18:19 ` Peter Zijlstra
2011-06-06 22:08 ` Ingo Molnar
2011-06-03 12:44 ` [tip:sched/locking] sched: Add p->pi_lock to task_rq_lock() Linus Torvalds
2011-06-03 13:05 ` Arne Jansen
2011-06-04 21:29 ` Linus Torvalds
2011-06-04 22:08 ` Peter Zijlstra
2011-06-04 22:50 ` Linus Torvalds
2011-06-05 6:01 ` Arne Jansen
2011-06-05 7:57 ` Mike Galbraith
2011-04-05 15:23 ` [PATCH 14/21] sched: Drop rq->lock from first part of wake_up_new_task() Peter Zijlstra
2011-04-14 8:37 ` [tip:sched/locking] " tip-bot for Peter Zijlstra
2011-04-05 15:23 ` [PATCH 15/21] sched: Drop rq->lock from sched_exec() Peter Zijlstra
2011-04-14 8:37 ` [tip:sched/locking] " tip-bot for Peter Zijlstra
2011-04-05 15:23 ` [PATCH 16/21] sched: Remove rq->lock from the first half of ttwu() Peter Zijlstra
2011-04-14 8:38 ` [tip:sched/locking] " tip-bot for Peter Zijlstra
2011-04-05 15:23 ` [PATCH 17/21] sched: Remove rq argument from ttwu_stat() Peter Zijlstra
2011-04-14 8:38 ` [tip:sched/locking] " tip-bot for Peter Zijlstra
2011-04-05 15:23 ` [PATCH 18/21] sched: Rename ttwu_post_activation Peter Zijlstra
2011-04-14 8:39 ` [tip:sched/locking] sched: Rename ttwu_post_activation() to ttwu_do_wakeup() tip-bot for Peter Zijlstra
2011-04-05 15:23 ` [PATCH 19/21] sched: Restructure ttwu some more Peter Zijlstra
2011-04-14 8:39 ` [tip:sched/locking] sched: Restructure ttwu() " tip-bot for Peter Zijlstra
2011-04-05 15:23 ` [PATCH 20/21] sched: Move the second half of ttwu() to the remote cpu Peter Zijlstra
2011-04-14 8:39 ` [tip:sched/locking] " tip-bot for Peter Zijlstra
2011-04-05 15:23 ` [PATCH 21/21] sched: Remove need_migrate_task() Peter Zijlstra
2011-04-14 8:40 ` [tip:sched/locking] " tip-bot for Peter Zijlstra
2011-04-05 15:59 ` [PATCH 00/21] sched: Reduce runqueue lock contention -v6 Peter Zijlstra
2011-04-06 11:00 ` Peter Zijlstra
2011-04-27 16:54 ` Dave Kleikamp
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=20110405152728.990364093@chello.nl \
--to=a.p.zijlstra@chello.nl \
--cc=axboe@kernel.dk \
--cc=chris.mason@oracle.com \
--cc=efault@gmx.de \
--cc=frank.rowand@am.sony.com \
--cc=linux-kernel@vger.kernel.org \
--cc=mingo@elte.hu \
--cc=oleg@redhat.com \
--cc=pjt@google.com \
--cc=tglx@linutronix.de \
--cc=yong.zhang0@gmail.com \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
@@ -2301,7 +2301,7 @@ void task_oncpu_function_call(struct tas
#ifdef CONFIG_SMP
/*
- * ->cpus_allowed is protected by either TASK_WAKING or rq->lock held.
+ * ->cpus_allowed is protected by both rq->lock and p->pi_lock
*/
static int select_fallback_rq(int cpu, struct task_struct *p)
@@ -2334,7 +2334,7 @@ static int select_fallback_rq(int cpu, s
}
/*
- * The caller (fork, wakeup) owns TASK_WAKING, ->cpus_allowed is stable.
+ * The caller (fork, wakeup) owns p->pi_lock, ->cpus_allowed is stable.
*/
static inline
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is an external index of several public inboxes,
see mirroring instructions on how to clone and mirror
all data and code used by this external index.